#include "Particle.h"

Particle::Particle()
{
	x = y = z = 0;
	speedx = speedy = speedz = 0;
	starttime = lifetime = Particle::IMMORTAL;
	r = .8;
	g = 0;
	b = .2;
}

Particle::Particle(float _x, float _y, float _z, float _lifetime)
{
	x = _x;
	y = _y;
	z = _z;
	speedx = speedy = speedz = 0;
	starttime = lifetime = _lifetime;
	r = .8;
	g = 0;
	b = .2;
}

void Particle::setColor(float _r, float _g, float _b)
{
	r = _r;
	g = _g;
	b = _b;
}

void Particle::draw()
{
	float alpha = lifetime/starttime * 1.5;
	if(alpha > 1)
		alpha = 1;
	glColor3f(r * alpha, g * alpha, b * alpha);
	glPushMatrix();
	glTranslatef(x, y, z);
	glutSolidSphere(.01, 3, 3);
	glPopMatrix();
}

void Particle::performTimestep(float alpha)
{
	x += speedx * alpha;
	y += speedy * alpha;
	z += speedz * alpha;
	if(lifetime > 0)
	{
		lifetime-= alpha;
	}
}

void Particle::addSpeed(float _x, float _y, float _z)
{
	speedx += _x;
	speedy += _y;
	speedz += _z;
}

bool Particle::isAlive()
{
	if(lifetime == Particle::IMMORTAL)
		return true;
	if(lifetime > 0)
		return true;
	return false;
}