#include "ParticleEngine.h"

ParticleEngine::ParticleEngine()
{
	gravx = 0;
	gravy = -1;
	gravz = 0;
	commonInit();
}

ParticleEngine::ParticleEngine(float _gravx, float _gravy, float _gravz)
{
	gravx = _gravx;
	gravy = _gravy;
	gravz = _gravz;
	commonInit();
}

void ParticleEngine::setGravity(float _x, float _y, float _z)
{
	gravx = _x;
	gravy = _y;
	gravz = _z;
}

void ParticleEngine::commonInit()
{
	srand(1);
	// insert 10 particles
	for(int i=0; i<10; i++)
	{
		createParticle();
	}
}

void ParticleEngine::performTimestep(float alpha)
{
	float jitterx, jitterz;

	// perform timestep for living particles
	for(std::list<Particle*>::iterator i = particleList.begin(); i != particleList.end(); ++i)
	{
		jitterx = ((rand() - (RAND_MAX/2)) /(float)RAND_MAX) *.1;
		jitterz = ((rand() - (RAND_MAX/2)) /(float)RAND_MAX) *.1;
		(*i)->addSpeed(gravx * alpha + jitterx, gravy * alpha, gravz * alpha + jitterz);
		(*i)->performTimestep(alpha);
		// erase dead particles
		if(!(*i)->isAlive())
		{
			delete (*i);
			i = particleList.erase(i);
		}
	}

	// create new particles
	for(int n=0; n<birtrate*alpha; n++)
	{
		createParticle();
	}
}

void ParticleEngine::createParticle()
{
		float x = rand() /(float)RAND_MAX;
		float z = rand() /(float)RAND_MAX;
		Particle *p = new Particle(x, 0, z, 5);
		if(rand() > (RAND_MAX/3))
			p->setColor(.9, .9, .1);
		
		particleList.push_back(p);
}

void ParticleEngine::draw()
{
	for(std::list<Particle*>::iterator i = particleList.begin(); i != particleList.end(); ++i)
		(*i)->draw();
}

void ParticleEngine::setBirthrate(int rate)
{
	birtrate = rate;
}