/* -----------------------------------------------------------	*/
/* Particle Engine Tutorial										*/
/* -----------------------------------------------------------	*/
/* by n0rmAnfreAK												*/
/* visit www.normanfreak.de.vu for updates						*/
/*																*/
/* feel free to email me any critic and comments				*/
/* ------------------------------------------------------------ */

#include <windows.h>
#include <gl\glut.h>
#include <stdio.h>
#include <math.h>

#include "ParticleEngine.h"

int frames;
long time, lastTime, animationTime;
char title[100];

float angleh, anglev;
ParticleEngine *particleEngine;

// all code that has to be run once before the rendering
void Init()	
{
	glClearColor(0.33f,0.225f,0.0f,0.0f);
	glClearStencil(0);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);	
	glShadeModel(GL_SMOOTH);
	//glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
	glClearDepth(1.0);

	angleh = anglev = 0.f;
	particleEngine = new ParticleEngine(0, .2, 0);
	particleEngine->setBirthrate(140);
	

	frames = 0;
	time = 0;
	lastTime = animationTime = GetTickCount();	
}

void Scene()
{
	
	glLoadIdentity ();
	gluLookAt(0, 1, -3, 0, 0, 1.2, 0, 1, 0); 

	// rotate by mousemovement
	glRotatef(anglev, 1, 0,	0);
	glRotatef(angleh, 0, 1, 0);

	particleEngine->draw();
}


void RenderScene(void)
{
	// clear all buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	// render Scene ambient
	Scene();

	glutSwapBuffers (); 
	glFlush ();

	// bruteforce to get fps-count
	frames++;
	long time = GetTickCount();
	if(time - lastTime > 1000)
	{
		lastTime = time;
		sprintf(title, "Particle Engine Tutorial  v0.3 - fps : %i", frames);
		glutSetWindowTitle(title);
		frames = 0;
	}
}

// all code to reshape the window
void Reshape(int width,int height)
{	
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	glViewport(0,0,width,height);
	
	GLfloat fAspect;
	if(height==0)
		height=1;
	fAspect=(float)width/(float)height;

	// clipping range !!!! WICHTIG ;)
	gluPerspective(60.0f, fAspect, 0.1, 300.0);
	glMatrixMode(GL_MODELVIEW); 
}

// all code to animate the scene is placed here

void Animate ()
{
	long time = GetTickCount();
	if(time - animationTime > 200)
	{
		animationTime = time;
		particleEngine->performTimestep(.1f);	
	}
	RenderScene();	
}


void Keyboard(unsigned char key, int x, int y)
{
	// perform keypressed here
	switch(key)
	{
	
	}
}

int oldX, oldY;
void MouseLook(int x, int y)
{
	angleh += (oldX - x)/2.0f;
	anglev += (oldY - y)/2.0f;
	RenderScene();
	oldX = x;
	oldY = y;
}

void MousePressed(int button, int state, int x, int y)
{
	if(button == GLUT_LEFT_BUTTON)
	{	
		oldX = x;
		oldY = y;
	}
}

void gc()
{
	printf(" clear all");
	delete particleEngine;
}

int main(int argc, char **argv)
{
	printf ("Particle Engine Tutorial \n\n");
	glutInit ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH|GLUT_STENCIL );
	glutInitWindowSize ( 600,600 );	
	glutCreateWindow ("Particle Engine v0.3");
	glutDisplayFunc ( RenderScene );
	glutReshapeFunc ( Reshape );
	glutIdleFunc ( Animate );
	glutKeyboardFunc ( Keyboard );
	glutMouseFunc ( MousePressed );
	glutMotionFunc ( MouseLook );
	Init();
	glutMainLoop ();
	gc();
	return 0;
}
